<?php

namespace Drupal\svg_sprite\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\svg_sprite\Service\SvgSpriteService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class SvgSpriteSettingsForm.
 *
 * The settings form of the svg_sprite module.
 *
 * @package Drupal\svg_sprite\Form
 */
class SvgSpriteSettingsForm extends SettingsFormBase {

  const SETTINGS = 'svg_sprite.settings';

  /**
   * The SVG Sprite service.
   *
   * @var \Drupal\svg_sprite\Service\SvgSpriteService
   */
  protected SvgSpriteService $svgSpriteService;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->svgSpriteService = $container->get('svg_sprite');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'svg_sprite_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getSettings() {
    return self::SETTINGS;
  }

  /**
   * {@inheritdoc}
   */
  protected function addFormElements(array &$form, $config) {

    $form['source'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Source of the SVG sprites file'),
      '#description' => $this->t("You can provide a full URL or specify a path relative to Drupal's web directory.<br/>You can also use the theme://[theme_name]/ prefix to target an SVG file in a theme."),
      '#default_value' => $config->get('source'),
      '#required' => TRUE,
    ];

    $form['alpha_sort'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Sort sprites alphabetically'),
      '#description' => $this->t("Will sort sprites alphabetically using the symbol's <code>id</code>, or <code>aria-label</code>, if provided.<br/>Leave unchecked to keep the ordering of the SVG source file."),
      '#default_value' => $config->get('alpha_sort'),
      '#required' => FALSE,
    ];

  }

  /**
   * {@inheritdoc}
   */
  protected function validateFormElements(FormStateInterface $form_state, $config) {
    $source = $form_state->getValue('source');
    $data = $this->svgSpriteService->fetchSvgDataFromSource($source);
    if (!$data) {
      $form_state->setErrorByName('source', $this->t('Invalid SVG source.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function saveFormElements(FormStateInterface $form_state, $config) {
    $source = $form_state->getValue('source');
    $alpha_sort = $form_state->getValue('alpha_sort');
    $data = $this->svgSpriteService->fetchSvgDataFromSource($source);
    $symbols = $this->svgSpriteService->extractSpriteInfoFromSvgData($data, $alpha_sort);
    $href = $this->svgSpriteService->getSvgHref($source);
    $config
      ->set('source', $source)
      ->set('alpha_sort', $alpha_sort)
      ->set('symbols', $symbols)
      ->set('href', $href)
      ->save();
  }

}
